@@ -2,11 +2,17 @@
 from datetime import timedelta
 from webob import cookies
 from webob.compat import text_
-from nose.tools import eq_
+from nose.tools import (eq_, assert_raises)
 import unittest
 from webob.compat import native_
 from webob.compat import PY3
 
+def setup_module(module):
+    cookies._should_raise = True
+
+def teardown_module(module):
+    cookies._should_raise = False
+
 def test_cookie_empty():
     c = cookies.Cookie() # empty cookie
     eq_(repr(c), '<Cookie: []>')
@@ -42,9 +48,9 @@ def test_cookie_complex():
 
 def test_cookie_complex_serialize():
     c = cookies.Cookie('dismiss-top=6; CP=null*, '\
-                       'PHPSESSID=0a539d42abc001cdc762809248d4beed, a="42,"')
+                       'PHPSESSID=0a539d42abc001cdc762809248d4beed, a="42"')
     eq_(c.serialize(),
-        'CP=null*; PHPSESSID=0a539d42abc001cdc762809248d4beed; a="42\\054"; '
+        'CP=null*; PHPSESSID=0a539d42abc001cdc762809248d4beed; a=42; '
         'dismiss-top=6')
 
 def test_cookie_load_multiple():
@@ -92,30 +98,37 @@ def test_ch_unquote():
     eq_(cookies._unquote(b'"hello world'), b'"hello world')
     eq_(cookies._unquote(b'hello world'), b'hello world')
     eq_(cookies._unquote(b'"hello world"'), b'hello world')
-    eq_(cookies._value_quote(b'hello world'), b'"hello world"')
+
+    # Spaces are not valid in cookies, we support getting them, but won't support sending them
+    assert_raises(ValueError, cookies._value_quote, b'hello world')
+
     # quotation mark escaped w/ backslash is unquoted correctly (support
     # pre webob 1.3 cookies)
     eq_(cookies._unquote(b'"\\""'), b'"')
     # we also are able to unquote the newer \\042 serialization of quotation
     # mark
     eq_(cookies._unquote(b'"\\042"'), b'"')
-    # but when we generate a new cookie, quote using normal octal quoting
-    # rules
-    eq_(cookies._value_quote(b'"'), b'"\\042"')
+
+    # New cookies can not contain quotes.
+    assert_raises(ValueError, cookies._value_quote, b'"')
+
     # backslash escaped w/ backslash is unquoted correctly (support
     # pre webob 1.3 cookies)
     eq_(cookies._unquote(b'"\\\\"'), b'\\')
     # we also are able to unquote the newer \\134 serialization of backslash
     eq_(cookies._unquote(b'"\\134"'), b'\\')
-    # but when we generate a new cookie, quote using normal octal quoting
-    # rules
-    eq_(cookies._value_quote(b'\\'), b'"\\134"')
+
+    # Cookies may not contain a backslash
+    assert_raises(ValueError, cookies._value_quote, b'\\')
+
     # misc byte escaped as octal
     eq_(cookies._unquote(b'"\\377"'), b'\xff')
-    eq_(cookies._value_quote(b'\xff'), b'"\\377"')
+
+    assert_raises(ValueError, cookies._value_quote, b'\xff')
+
     # combination
     eq_(cookies._unquote(b'"a\\"\\377"'), b'a"\xff')
-    eq_(cookies._value_quote(b'a"\xff'), b'"a\\042\\377"')
+    assert_raises(ValueError, cookies._value_quote, b'a"\xff')
 
 def test_cookie_invalid_name():
     c = cookies.Cookie()
@@ -144,11 +157,6 @@ def test_serialize_max_age_str():
     result = cookies.serialize_max_age(val)
     eq_(result, b'86400')
 
-def test_escape_comma_semi_dquote():
-    c = cookies.Cookie()
-    c['x'] = b'";,"'
-    eq_(c.serialize(True), r'x="\042\073\054\042"')
-
 def test_parse_qmark_in_val():
     v = r'x="\"\073\054\""; expires=Sun, 12-Jun-2011 23:16:01 GMT'
     c = cookies.Cookie(v)
@@ -229,19 +237,19 @@ class TestRequestCookies(unittest.TestCase):
         self.assertRaises(ValueError, inst.__setitem__, 'a', value)
 
     def test__setitem__success_no_existing_headers(self):
-        value = native_(b'La Pe\xc3\xb1a', 'utf-8')
+        value = native_(b'test_cookie', 'utf-8')
         environ = {}
         inst = self._makeOne(environ)
         inst['a'] = value
-        self.assertEqual(environ['HTTP_COOKIE'], 'a="La Pe\\303\\261a"')
+        self.assertEqual(environ['HTTP_COOKIE'], 'a=test_cookie')
 
     def test__setitem__success_append(self):
-        value = native_(b'La Pe\xc3\xb1a', 'utf-8')
+        value = native_(b'test_cookie', 'utf-8')
         environ = {'HTTP_COOKIE':'a=1; b=2'}
         inst = self._makeOne(environ)
         inst['c'] = value
         self.assertEqual(
-            environ['HTTP_COOKIE'], 'a=1; b=2; c="La Pe\\303\\261a"')
+            environ['HTTP_COOKIE'], 'a=1; b=2; c=test_cookie')
 
     def test__setitem__success_replace(self):
         environ = {'HTTP_COOKIE':'a=1; b="La Pe\\303\\261a"; c=3'}
@@ -361,6 +369,20 @@ class CookieMakeCookie(unittest.TestCase):
         self.assertTrue('test_cookie=value' in cookie)
         self.assertTrue('Max-Age=500;' in cookie)
         self.assertTrue('expires' in cookie)
+        self.assertFalse('expires=500' in cookie)
+
+    def test_make_cookie_max_age_str_valid_int(self):
+        cookie = self.makeOne('test_cookie', 'value',
+                              max_age='500')
+
+        self.assertTrue('test_cookie=value' in cookie)
+        self.assertTrue('Max-Age=500;' in cookie)
+        self.assertTrue('expires' in cookie)
+        self.assertFalse('expires=500' in cookie)
+
+    def test_make_cookie_max_age_str_invalid_int(self):
+        self.assertRaises(ValueError, self.makeOne, 'test_cookie', 'value',
+                              max_age='test')
 
     def test_make_cookie_comment(self):
         cookie = self.makeOne('test_cookie', 'value', comment='lolwhy')
@@ -431,6 +453,22 @@ class CookieProfileTest(CommonCookieProfile):
         cookie = self.makeOne(serializer=RaisingSerializer())
         self.assertEqual(cookie.get_value(), None)
 
+    def test_with_cookies(self):
+        request = self.makeOneRequest()
+        request.cookies['uns'] = 'InRlc3Qi'
+        cookie = self.makeOne(request=request)
+        ret = cookie.get_value()
+
+        self.assertEqual(ret, "test")
+
+    def test_with_invalid_cookies(self):
+        request = self.makeOneRequest()
+        request.cookies['uns'] = 'InRlc3Q'
+        cookie = self.makeOne(request=request)
+        ret = cookie.get_value()
+
+        self.assertEqual(ret, None)
+
 class SignedCookieProfileTest(CommonCookieProfile):
     def makeOne(self, secret='seekrit', salt='salty', name='uns', **kw):
         if 'request' in kw:
