@@ -108,7 +108,7 @@ def deprecated_property(attr, name, text, version): # pragma: no cover
     """
     def warn():
         warn_deprecation('The attribute %s is deprecated: %s'
-            % (attr, text),
+            % (name, text),
             version,
             3
         )
@@ -122,7 +122,7 @@ def deprecated_property(attr, name, text, version): # pragma: no cover
         warn()
         attr.__delete__(self)
     return property(fget, fset, fdel,
-        '<Deprecated attribute %s>' % attr
+        '<Deprecated attribute %s>' % name
     )
 
 
@@ -303,7 +303,7 @@ def serialize_content_range(value):
 
 
 
-_rx_auth_param = re.compile(r'([a-z]+)=(".*?"|[^,]*)(?:\Z|, *)')
+_rx_auth_param = re.compile(r'([a-z]+)[ \t]*=[ \t]*(".*?"|[^,]*?)[ \t]*(?:\Z|, *)')
 
 def parse_auth_params(params):
     r = {}
