@@ -1,5 +1,4 @@
 import binascii
-import cgi
 import io
 import os
 import re
@@ -39,6 +38,7 @@ from webob.compat import (
     url_unquote,
     quote_plus,
     urlparse,
+    cgi_FieldStorage
     )
 
 from webob.cookies import RequestCookies
@@ -229,13 +229,13 @@ class BaseRequest(object):
         fs_environ.setdefault('CONTENT_LENGTH', '0')
         fs_environ['QUERY_STRING'] = ''
         if PY3: # pragma: no cover
-            fs = cgi.FieldStorage(fp=self.body_file,
+            fs = cgi_FieldStorage(fp=self.body_file,
                                   environ=fs_environ,
                                   keep_blank_values=True,
                                   encoding=charset,
                                   errors=errors)
         else:
-            fs = cgi.FieldStorage(fp=self.body_file,
+            fs = cgi_FieldStorage(fp=self.body_file,
                                   environ=fs_environ,
                                   keep_blank_values=True)
 
@@ -775,7 +775,7 @@ class BaseRequest(object):
             if body_file is self.body_file_raw:
                 return vars
         content_type = self.content_type
-        if ((self.method == 'PUT' and not content_type)
+        if ((self.method != 'POST' and not content_type)
             or content_type not in
                 ('',
                  'application/x-www-form-urlencoded',
@@ -785,32 +785,29 @@ class BaseRequest(object):
             return NoVars('Not an HTML form submission (Content-Type: %s)'
                           % content_type)
         self._check_charset()
-        if self.is_body_seekable:
-            self.body_file_raw.seek(0)
+
+        self.make_body_seekable()
+        self.body_file_raw.seek(0)
+
         fs_environ = env.copy()
         # FieldStorage assumes a missing CONTENT_LENGTH, but a
         # default of 0 is better:
         fs_environ.setdefault('CONTENT_LENGTH', '0')
         fs_environ['QUERY_STRING'] = ''
         if PY3: # pragma: no cover
-            fs = cgi.FieldStorage(
+            fs = cgi_FieldStorage(
                 fp=self.body_file,
                 environ=fs_environ,
                 keep_blank_values=True,
                 encoding='utf8')
             vars = MultiDict.from_fieldstorage(fs)
         else:
-            fs = cgi.FieldStorage(
+            fs = cgi_FieldStorage(
                 fp=self.body_file,
                 environ=fs_environ,
                 keep_blank_values=True)
             vars = MultiDict.from_fieldstorage(fs)
 
-
-        #ctype = self.content_type or 'application/x-www-form-urlencoded'
-        ctype = self._content_type_raw or 'application/x-www-form-urlencoded'
-        f = FakeCGIBody(vars, ctype)
-        self.body_file = io.BufferedReader(f)
         env['webob._parsed_post_vars'] = (vars, self.body_file_raw)
         return vars
 
@@ -1146,7 +1143,7 @@ class BaseRequest(object):
         # acquire body before we handle headers so that
         # content-length will be set
         body = None
-        if self.method in ('PUT', 'POST'):
+        if http_method_probably_has_body.get(self.method):
             if skip_body > 1:
                 if len(self.body) > skip_body:
                     body = bytes_('<body skipped (len=%s)>' % len(self.body))
@@ -1248,7 +1245,7 @@ class BaseRequest(object):
             if hname in r.headers:
                 hval = r.headers[hname] + ', ' + hval
             r.headers[hname] = hval
-        if r.method in ('PUT', 'POST'):
+        if http_method_probably_has_body.get(r.method):
             clen = r.content_length
             if clen is None:
                 body = fp.read()
@@ -1566,7 +1563,7 @@ def _cgi_FieldStorage__repr__patch(self):
         return "FieldStorage(%r, %r)" % (self.name, self.filename)
     return "FieldStorage(%r, %r, %r)" % (self.name, self.filename, self.value)
 
-cgi.FieldStorage.__repr__ = _cgi_FieldStorage__repr__patch
+cgi_FieldStorage.__repr__ = _cgi_FieldStorage__repr__patch
 
 class FakeCGIBody(io.RawIOBase):
     def __init__(self, vars, content_type):
